
#include <u.h>

extern int ofcall(void *);

static int stdout = 0;
static char *logbuf = (char*)0x2000;
static int logpos = 0;
enum { TxReady = 1<<2 };
typedef struct {
	struct {
		uchar	csr;
		uchar	dummy1;
		uchar	dr;
		uchar	dummy2;
	} *ptr;
} Ctrl;
static Ctrl dev = { (void*)0xfffa6004 };


int
offinddevice(char *device)
{
	static struct {
		int p0;
		char *name;
		int p1, nargs;
		int p2, nreturns;
		int p3, device;
		int p4, phandle;
	} args = {
		0, "finddevice", 0, 1, 0, 1
	};

	args.device = (int)device;
	ofcall(&args);
	return args.phandle;
}

int
ofgetprop(int handle, char *prop, void *buf, int buflen)
{
	static struct {
		int p0;
		char *name;
		int p1, nargs;
		int p2, nreturns;
		int p3, phandle;
		int p4, prop;
		int p5, buf;
		int p6, buflen;
		int p7, size;
	} args = {
		0, "getprop", 0, 4, 0, 1
	};

	args.phandle = handle;
	args.prop = (int)prop;
	args.buf = (int)buf;
	args.buflen = buflen;
	ofcall(&args);
	return args.size;
}

int
ofwrite(int handle, void *addr, int len)
{
	static struct {
		int p0;
		char *name;
		int p1, nargs;
		int p2, nreturns;
		int p3, ihandle;
		int p4, addr;
		int p5, len;
		int p6, actual;
	} args = {
		0, "write", 0, 3, 0, 1
	};

	args.ihandle = handle;
	args.addr = (int)addr;
	args.len = len;
	ofcall(&args);
	return args.actual;
}

static void
ofputc(char c)
{
	int dev;

	if(!stdout) {
		dev = offinddevice("/chosen");
		ofgetprop(dev, "stdout", &stdout, sizeof stdout);
		ofwrite(stdout, "\r\n", 2);
	}
	ofwrite(stdout, &c, 1);
}


static uchar
csr8r(Ctrl *cp, int addr)
{
	cp->ptr->csr = addr;
	return cp->ptr->csr;
}

static void
uartputc(char c)
{
	while(!(csr8r(&dev, 0) & TxReady))
		continue;
	dev.ptr->dr = c;
}

static void
logputc(char c)
{
	logbuf[logpos] = c;
	logpos = (logpos + 1) & 0xfff;
}


/* pick which we want to use */
static void (*putcfunc)(char) = uartputc;
static int putcCR = 1;

static void
printint(uint val, int base)
{
	static char *valch = "0123456789abcdef";
	int x, b;

	for(b = 1; val / b > base; b = b * base)
		continue;
	while(b) {
		x = val / b;
		val -= x * b;
		b /= base;
		putcfunc(valch[x]);
	}
}

void
xprint(char *fmt, ...)
{
	char *str, ch;
	va_list arg;

	va_start(arg, fmt);
	while(*fmt) {
		switch(*fmt) {
		case '%':
			fmt++;
			switch(*fmt) {
			case '\0':
				fmt--;
				break;
			case 'c':
				ch = va_arg(arg, int);
				putcfunc(ch);
				break;
			case 'd':
				printint(va_arg(arg, int), 10);
				break;
			case 's':
				str = va_arg(arg, char*);
				while(str)
					putcfunc(*str++);
				break;
			case 'x':
				printint(va_arg(arg, int), 16);
				break;
			default:
				putcfunc(*fmt);
				break;
			}
			break;
		case '\n':
			if(putcCR)
				putcfunc('\r');
			putcfunc('\n');
			break;
		default:
			putcfunc(*fmt);
			break;
		}
		fmt++;
	}
	va_end(arg);
}

void
main(void)
{
	putcfunc = uartputc;
	putcCR = 1;

	xprint("done\n");
}

